<?php

declare(strict_types=1);

namespace Drupal\image_processor_webp\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Image Processor settings.
 */
class ImageProcessorSettingsForm extends ConfigFormBase {

  public const LIBRARY_GD = 'gd';
  public const LIBRARY_IMAGEMAGICK = 'imagemagick';

  public const FORMAT_WEBP = 'webp';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'image_processor_webp_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['image_processor_webp.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('image_processor_webp.settings');

    $form['preprocess_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable image processing'),
      '#description' => $this->t('Check to enable automatic image processing.'),
      '#default_value' => $config->get('preprocess_enabled'),
    ];

    $form['preprocess_library'] = [
      '#type' => 'select',
      '#title' => $this->t('Image processing library'),
      '#description' => $this->t('Select which library to use for image processing.'),
      '#options' => [
        self::LIBRARY_GD => $this->t('GD Library'),
        self::LIBRARY_IMAGEMAGICK => $this->t('ImageMagick'),
      ],
      '#default_value' => $config->get('preprocess_library'),
    ];

    $form['format'] = [
      '#type' => 'select',
      '#title' => $this->t('Image format'),
      '#description' => $this->t('Select format to use for image processing.'),
      '#options' => [
        self::FORMAT_WEBP => $this->t('Webp'),
      ],
      '#default_value' => $config->get('format'),
    ];

    $form['webp_quality'] = [
      '#type' => 'number',
      '#title' => $this->t('WEBP Image Quality'),
      '#description' => $this->t('Define the quality for WEBP images, range from 0 (worst quality, smallest file) to 100 (best quality, largest file).'),
      '#default_value' => $config->get('webp_quality') ?? 80,
      '#min' => 0,
      '#max' => 100,
      '#step' => 1,
    ];

    $form['log_missing_images'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log missing images'),
      '#description' => $this->t('Enable this option to log missing WebP images for further processing and optimization.'),
      '#default_value' => $config->get('log_missing_images'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('format') === 'avif' && $form_state->getValue('preprocess_library') === self::LIBRARY_GD) {
      $form_state->setErrorByName('format', $this->t('AVIF is not supported with GD library.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('image_processor_webp.settings')
      ->set('preprocess_library', $form_state->getValue('preprocess_library'))
      ->set('preprocess_enabled', $form_state->getValue('preprocess_enabled'))
      ->set('format', $form_state->getValue('format'))
      ->set('webp_quality', $form_state->getValue('webp_quality'))
      ->set('log_missing_images', $form_state->getValue('log_missing_images'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
