<?php

declare(strict_types=1);

namespace Drupal\image_processor_webp;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\file\Entity\File;
use Drupal\image_processor_webp\Form\ImageProcessorSettingsForm;

/**
 * Handles image conversion to WebP format based on configuration.
 */
class ImageProcessor {

  /**
   * Configuration settings for image processing.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $config;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  private LoggerChannelInterface $logger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private FileSystemInterface $fileSystem;

  /**
   * Constructs an ImageProcessor object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelInterface $logger, FileSystemInterface $file_system) {
    $this->config = $config_factory->get('image_processor_webp.settings');
    $this->logger = $logger;
    $this->fileSystem = $file_system;
  }

  /**
   * Converts the provided image file to WebP format.
   *
   * @param \Drupal\file\Entity\File $file
   *   The file entity to be converted.
   */
  public function convertToWebp(File $file): void {
    if (!$this->config->get('preprocess_enabled')) {
      return;
    }

    $library = $this->config->get('preprocess_library') ?: ImageProcessorSettingsForm::LIBRARY_GD;

    if ($library === ImageProcessorSettingsForm::LIBRARY_GD) {
      $this->convertGdImageWebp($file);
    }
    elseif ($library === ImageProcessorSettingsForm::LIBRARY_IMAGEMAGICK) {
      $this->convertImageMagicImageWebp($file);
    }
  }

  /**
   * Converts an image to WebP format using the GD library.
   *
   * @param \Drupal\file\Entity\File $file
   *   The file entity to be converted.
   */
  private function convertGdImageWebp(File $file): void {
    if (!function_exists('imagewebp')) {
      $this->logger->error('GD library does not support WebP format.');
      return;
    }

    $image = $this->createImageResource($file->getFileUri(), $file->getMimeType());
    if (!$image) {
      $this->logger->error('Failed to create image resource from file: @uri', ['@uri' => $file->getFileUri()]);
      return;
    }

    $quality = $this->config->get('webp_quality') ?: 80;
    $path_parts = pathinfo($file->getFileUri());
    $webp_path = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.webp';

    try {
      if (!imagewebp($image, $webp_path, (int) $quality)) {
        $this->logger->error('Failed to convert to WebP: @uri', ['@uri' => $file->getFileUri()]);
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to convert to WebP: @uri', ['@uri' => $file->getFileUri()]);
    }

    imagedestroy($image);
  }

  /**
   * Converts an image to WebP format using the ImageMagick library.
   *
   * @param \Drupal\file\Entity\File $file
   *   The file entity to be converted.
   */
  private function convertImageMagicImageWebp(File $file): void {
    $imagick = new \Imagick($file->getFileUri());

    try {
      $imagick->setImageFormat('webp');
      $quality = $this->config->get('webp_quality') ?: 80;
      $imagick->setImageCompressionQuality((int) $quality);

      $path_parts = pathinfo($file->getFileUri());
      $webp_path = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.webp';
      $realpath = $this->fileSystem->realpath($webp_path);

      if (!$imagick->writeImage($realpath)) {
        $this->logger->error('Failed to convert to WebP using ImageMagick: @uri', ['@uri' => $file->getFileUri()]);
      }
    }
    catch (\ImagickException $e) {
      $this->logger->error('ImageMagick exception on converting to WebP: @error', ['@error' => $e->getMessage()]);
    }
    finally {
      $imagick->clear();
      $imagick->destroy();
    }
  }

  /**
   * Creates an image resource based on the MIME type of the file.
   *
   * Supported formats are JPEG and PNG.
   *
   * @param string $uri
   *   The file URI.
   * @param string $type
   *   The MIME type of the image.
   *
   * @return resource|null
   *   The image resource or NULL if the MIME type is not supported.
   */
  private function createImageResource(string $uri, string $type) {
    switch ($type) {
      case 'image/jpeg':
        return imagecreatefromjpeg($uri);

      case 'image/png':
        return imagecreatefrompng($uri);

      default:
        return NULL;
    }
  }

}
